# -*- coding: utf-8 -*-
# klassendiagramm.py
"""StartApp und Controller fuer Klassendiagramme"""

import os.path

import wx
import pickle # Persistenz realisieren

from klassendiagramme.gui import Gui
from klassendiagramme.grafikfenster import *
from klassendiagramme.figuren import *
from klassendiagramme.klassenkarte import Klassenkarte
from klassendiagramme.verbinder import *

### ---------------------------------------------------
class Klassendiagramm():
    """Klassendiagramm (UML)"""

    def __init__(self, gui, diagrammFenster):
        self.__gui=gui
        self.__fenster=diagrammFenster
        self.__klassenkarten = []
        self.__assoziationen = []
        self.__vererbungen = []
        self.__ausgewaehlt = None
        self.__mausPosBeimZiehen = None
        self.__geaendert = False
        self.__projektName=None

    def IstGeaendert(self): return self.__geaendert
    def HatDaten(self):
        '''gibt zurueck, ob bereits ein Modell existiert'''
        if self.__klassenkarten != [] : return True
        if self.__assoziationen != [] : return True
        if self.__vererbungen != [] : return True
        return False

    def GibKlassenNamen(self):
        namen=[]
        for karte in self.__klassenkarten:
            namen.append(karte.GibNamen())
        return namen

    def GibKarteZuNamen(self, name):
        for karte in self.__klassenkarten:
            if name==karte.GibNamen(): return karte
        return None

    def GibAusgewaehlte(self): return self.__ausgewaehlt
    def NichtsAusgewaehlt(self): return self.__ausgewaehlt==None
    def Waehle(self, karte=''):
        '''wechselt die ausgewaehlte Karte, ggf auf die uebergebene'''
        if (karte==None) or (len(self.__klassenkarten))==0:
            self.__ausgewaehlt=None
            self.__gui.SetzeBeschriftungKlasse(u"keine Karte ausgewählt","","")
            return
        if karte=='':
            ausgewaehlt=self.GibAusgewaehlte()
            if self.__ausgewaehlt==None:  # Auswahl starten
                self.__ausgewaehlt=self.__klassenkarten[0]
            else:                         # Auswahl weiter setzen
                index=self.__klassenkarten.index(self.__ausgewaehlt)
                self.__ausgewaehlt=self.__klassenkarten[(index+1)%len(self.__klassenkarten)]
            karte=self.__ausgewaehlt
        elif karte in self.__klassenkarten:
            self.__ausgewaehlt=karte
        self.__gui.SetzeBeschriftungKlasse(karte.GibNamen(),str(karte.GibX()),str(karte.GibY()))
            

    def NeueKlasse(self, name):
        if name==None: return
        karte=Klassenkarte(name, 100, 100, [],[]) # attribute, methoden
        self.__klassenkarten.append(karte)
        self.__ausgewaehlt=karte
        self.__geaendert=True
        return karte

    def AendereNamen(self, alterName, neuerName):
        karte=self.GibKarteZuNamen(alterName)
        karte.Verberge()
        karte.AendereNamen(neuerName)
        karte.Zeige()

    def NeuesAttribut(self, name, typ=None, isAbstract=False, isClassVar=False):
        if self.NichtsAusgewaehlt(): return
        karte=self.__ausgewaehlt
        karte.NeuesAttribut(name, typ, isAbstract, isClassVar)
        self.__geaendert=True

    def LoescheAttribut(self, name):
        if self.NichtsAusgewaehlt(): return
        karte=self.__ausgewaehlt
        karte.LoescheAttribut(name)
        self.__geaendert=True

    def NeueMethode(self, name, typ=None, par=[],
                    isAbstract=False, isClassMeth=False):
        if self.NichtsAusgewaehlt(): return
        karte=self.__ausgewaehlt
        karte.NeueMethode(name, typ, par, isAbstract,isClassMeth)
        self.__geaendert=True

    def LoescheMethode(self, name):
        if self.NichtsAusgewaehlt(): return
        karte=self.__ausgewaehlt
        karte.LoescheMethode(name)
        self.__geaendert=True

    def NeueAssoziation(self, start,startkante,ziel,zielkante,name='',typ=0,verbinderTyp=0,richtung=0):
        if None in [start,startkante,ziel,zielkante]: return
        startkarte,zielkarte=self.NameOderKarte(start,ziel)
        asso = Assoziation(startkarte,startkante,zielkarte,zielkante,typ=typ,verbindertyp=verbinderTyp)
        ##asso.SetzeVerbinderTyp(verbinderTyp)
        self.__assoziationen.append(asso)
        self.__geaendert=True
        return asso

    def NeueVererbung(self, start,startkante,ziel,zielkante,name='',verbinderTyp=0):
        if None in [start,startkante,ziel,zielkante]: return
        startkarte,zielkarte=self.NameOderKarte(start,ziel)
        pfeil = VererbungsPfeil(startkarte,startkante,zielkarte,zielkante,name,verbindertyp=verbinderTyp)
        self.__vererbungen.append(pfeil)
        self.__geaendert=True
        return pfeil

    def NameOderKarte(self,start,ziel):
        '''gibt den benoetigten Typ Karte zurueck'''
        if start.__class__!=Klassenkarte: # ungleiche Typen fuehren zu Fehler
            startkarte=self.GibKarteZuNamen(start)
            zielkarte=self.GibKarteZuNamen(ziel)
        else:
            startkarte=start
            zielkarte=ziel
        return startkarte,zielkarte

    def GibVerbinder(self):
        '''gibt alle Verbinder zurueck'''
        return self.__assoziationen+self.__vererbungen

    def GibVerbinderMitTyp(self, die_kd_verbinder=None):
        '''gibt alle (passenden) Verbinder textlich zurueck'''
        if die_kd_verbinder==None:
            die_kd_verbinder=self.GibVerbinder()
        verbinderTexte = []
        for v in die_kd_verbinder:
            verbinderTexte.append(v.GibStartKarte().GibNamen()+' '+
                                  v.GibZielKarte().GibNamen()+' '+
                                  v.GibTypNamen())
        return verbinderTexte

    def LoescheVerbinder(self, verbinderTextMitTyp):
        start,ziel,typ=verbinderTextMitTyp
        startkarte,zielkarte=self.NameOderKarte(start,ziel)
        if typ in ['Assoziation','Aggregation','Komposition']:
            for asso in self.__assoziationen:
                if asso.GibStartKarte()==startkarte and\
                   asso.GibZielKarte()==zielkarte:
                    asso.Verberge()
                    self.__assoziationen.remove(asso)
        if typ=='Vererbung':
            for v in self.__vererbungen:
                if v.GibStartKarte()==startkarte and\
                   v.GibZielKarte()==zielkarte:
                    v.Verberge()
                    self.__vererbungen.remove(v)
        self.__geaendert=True

    def GibKarteZuNummer(self, nummer):
        if nummer < 1: return None
        if nummer > len(self.__klassenkarten): return None
        return self.__klassenkarten[nummer-1]

    def GibKarteZuName(self, name):
        for karte in self.__klassenkarten:
            if karte.GibNamen()==name:
                return karte
        return None

    def BildExport(self, dateiname='Klassendiagramm.png'):
        '''exportiert das Bild des Klassendiagramms'''
        Zeichenflaeche.GibZeichenflaeche().BildExport(dateiname)

    def ImportiereKlasse(self, klassenName):   ## ToDo
        ##Klasse=eval(klassenName)
        daten=Klasse.__dict__
        print(daten)

    #### ----- sichern und laden -----
    def GibDaten(self):
        return self.__modellDaten

    def ZeigeDaten(self):
        '''Modelldaten zeilenweise lesbar'''
        daten=self.__modellDaten
        zeilen=['Klassen:']
        index=0
        while (index<len(daten)) and (daten[index][0]=='Klasse'):
            zeilen.append(daten[index][1])
            index+=1
        zeilen.append('\nAssoziationen:')
        if (index<=len(daten)) and (daten[index][0]=='Vererbung'):
            zeilen.append('- - -')
        while (index<len(daten)) and (daten[index][0]=='Assoziation'):
            zeilen.append(daten[index][1])
            index+=1
        zeilen.append('\nVererbung:')
        if (index>len(daten)):
            zeilen.append('- - -')
        while (index<len(daten)) and (daten[index][0]=='Vererbung'):
            zeilen.append(daten[index][1])
            index+=1
        return zeilen
    
    def SichereDaten(self):
        self.__modellDaten=[]
        for karte in self.__klassenkarten:
            self.__modellDaten.append(('Klasse',karte.GibDatenTextlich()))
        for asso in self.__assoziationen:
            self.__modellDaten.append(('Assoziation',asso.GibDatenTextlich()))
        for vererbung in self.__vererbungen:
            self.__modellDaten.append(('Vererbung',vererbung.GibDatenTextlich()))
        return self.__modellDaten
        
    def ErzeugeModellAusDaten(self, modellDaten):
        for daten in modellDaten:
            if daten[0]=='Klasse':
                name=daten[1][0]
                karte=self.NeueKlasse(name)
                x,y,b,h=eval(daten[1][1])
                karte.Bewege(x-karte.GibX(),y-karte.GibY())
                karte.NeueBreite(b)
                abstrakt=eval(str(daten[1][4])) # bool sicherstellen
                karte.SetzeAbstrakt(abstrakt)
                for attribut in eval(daten[1][2]):
                    if len(attribut)==2:
                        karte.NeuesAttribut(attribut[0],attribut[1])
                    elif len(attribut)==3:
                        karte.NeuesAttribut(attribut[0],attribut[1],attribut[2])
                    else:
                        karte.NeuesAttribut(attribut[0],attribut[1],attribut[2],attribut[3])
                for methode in eval(daten[1][3]):                    
                    if len(methode)==3:
                        karte.NeueMethode(methode[0],methode[1],methode[2])
                    elif len(methode)==4:
                        karte.NeueMethode(methode[0],methode[1],methode[2],methode[3])
                    else:
                        karte.NeueMethode(methode[0],methode[1],methode[2],methode[3],methode[4])
            elif daten[0]=='Assoziation':
                asso=self.NeueAssoziation(daten[1][0],      # start
                                          daten[1][1],      # startkante
                                          daten[1][3],      # ziel
                                          daten[1][4],      # zielkante
                                          daten[1][6],      # name
                                          int(daten[1][7]), # Typ Assoziation
                                          int(daten[1][8]), # VerbinderTyp
                                          int(daten[1][9])  # Richtung
                                          )
                if int(daten[1][2])!=50:
                    asso.NeuerTeilerStartKante(int(daten[1][2]))
                if int(daten[1][5])!=50:
                    asso.NeuerTeilerZielKante(int(daten[1][5]))
                if int(daten[1][9])!=0:
                    asso.SetzeRichtung(int(daten[1][9]))
            elif daten[0]=='Vererbung':
                pfeil=self.NeueVererbung(daten[1][0],      # start
                                         daten[1][1],      # startkante
                                         daten[1][3],      # ziel
                                         daten[1][4],      # zielkante
                                         daten[1][6],      # name
                                         int(daten[1][8]), # VerbinderTyp
                                         )                 # typ ist hier statisch 3
                if int(daten[1][2])!=50:
                    pfeil.NeuerTeilerStartKante(int(daten[1][2]))
                if int(daten[1][5])!=50:
                    pfeil.NeuerTeilerZielKante(int(daten[1][5]))
            else:
                self.__gui.Meldung('Fehler bei Modelldefinition: ')
            self.__geaendert=False

    #### Abschnitt für das Laden und Speichern von Modelldaten -----------------
    def Speichere(self, dateiname, modellDaten=[]):
        '''Die Modell - Daten werden in eine Datei gespeichert.'''
        self.SichereDaten()
        if self.__modellDaten==None: return
        try:
            f=open(dateiname,'w')
        except IOError as e:
            self.__gui.Ausgabe('Datei existiert nicht, keine Daten geschrieben')
            return None
        f.write(str(self.__modellDaten))
        f.close()
        self.__geaendert=False
        self.HoleProjektNamen(dateiname)
        return 'OK'

    def SpeichereOhne(self, dateiname, modellDaten=[]):
        '''Die Modell - Daten werden ohne Attribute und Methoden
        in eine Datei gespeichert.  Problem: 
        Die Breite sollte wieder auf Standarbreite gebracht werden'''
        self.SichereDaten()
        if self.__modellDaten==None: return
        self.ohne=[]
        for dat in self.__modellDaten:
            if dat[0]=='Klasse':
                xybh=eval(dat[1][1])
                abmessungen=str((xybh[0],xybh[1],80,xybh[3]))
                inhalt=(dat[1][0],abmessungen,'[]','[]')
                self.ohne.append((dat[0],inhalt))
            else:
                self.ohne.append(dat)
        try:
            f=open(dateiname,'w')
        except IOError as e:
            self.__gui.Ausgabe('Datei existiert nicht, keine Daten geschrieben')
            return None
        f.write(str(self.ohne))
        f.close()
        return 'OK'

    def Lade(self, dateiname):
        '''laedt die Daten aus der Datei'''
        try:
            f=open(dateiname,'r')
        except IOError as e:
            self.__gui.Ausgabe('Datei existiert nicht, keine Daten gelesen')
            return None
        modellDatenString=f.read()
        f.close()
        self.LoescheAktuellesModell()
        try:
            self.__modellDaten=eval(modellDatenString)
            self.ErzeugeModellAusDaten(self.__modellDaten)
            self.HoleProjektNamen(dateiname)
        except SyntaxError as e:
            self.__gui.Meldung('Fehler bei Modelldefinition: ')
            self.__modellDaten=[]
        return self.__modellDaten

    def HoleProjektNamen(self, dateiname):
        '''extrahiert den Projektnamen aus dem Dateinamen'''
        anfang=0
        for i in range(len(dateiname)):
            if dateiname[i] in ('/','\\'): anfang=i
        dateiname=dateiname[(anfang+1):]
        self.__projektName=dateiname.partition('.')[0]

    def GibProjektNamen(self):
        return self.__projektName

    def LoescheAktuellesModell(self):
        for asso in self.__assoziationen: asso.Verberge()
        for vererbung in self.__vererbungen: vererbung.Verberge()
        for karte in self.__klassenkarten: karte.Verberge()
        self.__klassenkarten = []
        self.__assoziationen = []
        self.__vererbungen = []
        self.__geaendert=False
        self.__projektName=None

    def GibVerbinderZu(self, karte):
        alleVerbinder=[]
        for verbinder in self.__assoziationen+self.__vererbungen:
            if verbinder.GibStartKarte()==karte:
                alleVerbinder.append(verbinder)
            elif verbinder.GibZielKarte()==karte:
                alleVerbinder.append(verbinder)
        return alleVerbinder
        
    #### ----- Mausereignisse -----
    def Enthalten(self,anX,anY,x,y,b,h):
        '''die Hilfsfunktion prueft, ob die Koordinaten
        im von den Daten gegebenen Rechteck liegen'''
        if anX<x: return False
        if anX>x+b: return False
        if anY<y: return False
        if anY>y+h: return False
        return True
        
    def Angeklickt(self,anX,anY):
        '''verarbeitet alle Mausklicks zum Auswaehlen'''
        for karte in self.__klassenkarten:
            x,y,b,h=karte.GibAbmessungen()
            if self.Enthalten(anX,anY,x,y,b,h):
                self.Waehle(karte)
                return karte
        ## sonst Auswahl loeschen
        self.Waehle(None)
        return None

    def AngeklickterVerbinder(self,anX,anY):
        '''verarbeitet rechten Mausklick zum Auswaehlen eines Verbinders
        gibt den Verbinder und Start(True) oder Ziel(False) zurueck'''
        for kd_v in self.__assoziationen+self.__vererbungen:
            verb=kd_v.GibVerbinder()
            x0,y0,x1,y1=verb.GibDaten()
            if self.Enthalten(anX,anY,x0-10,y0-10,20,20):
                return kd_v,True
            if self.Enthalten(anX,anY,x1-10,y1-10,20,20):
                return kd_v,False
        return None,None

    def MausClick(self,anX,anY):
        '''Waehlt aus und speichert ggf die Klickposition fuer das Ziehen'''
        self.__mausPosBeimZiehen = (anX,anY)
        if self.Angeklickt(anX,anY)==None:
            self.__mausPosBeimZiehen = None

    def MausLosgelassen(self,anX,anY):
        '''loescht die Mausposition vom Ziehen'''
        self.__mausPosBeimZiehen = None
        self.UpdateGrafikfenster()

    def DoppelClick(self,anX,anY):
        '''oeffnet den Konfigurationsdialog fuer
        Attribute und Methoden'''
        if self.Angeklickt(anX,anY)==None: return
        was=self.__gui.Auswahldialog('alternativ:',['Attribut','Methode'])
        if was=='Attribut': self.__gui.OnAttributBearbeiten()
        elif was=='Methode': self.__gui.OnMethodeBearbeiten()
        else: return
        
    def MausGezogen(self,anX,anY):
        '''verarbeitet das Ziehen einer Karte'''
        if self.NichtsAusgewaehlt(): return
        xM,yM = self.__mausPosBeimZiehen
        dx,dy = anX-xM,anY-yM
        self.GibAusgewaehlte().Bewege(dx,dy)
        self.__mausPosBeimZiehen = (anX,anY)
        self.UpdateGrafikfenster()
        
    def MausRechtsClick(self,anX,anY):
        '''oeffnet den Konfigurationsdialog fuer die
        Daten der Klassenkarte: xPosition,yPosition,Breite
        sowie alternativ fuer die Verbinder'''
        if self.Angeklickt(anX,anY)!=None:
            self.__gui.OnKlasseBearbeiten()
            return
        kd_verbinder,istStart=self.AngeklickterVerbinder(anX,anY)
        if kd_verbinder!=None:
            if istStart:
                alle=self.GibVerbinderZu(kd_verbinder.GibStartKarte())
            else:
                alle=self.GibVerbinderZu(kd_verbinder.GibZielKarte())
            self.__gui.VerbinderKonfigurieren(alle)

    def VerbergeBisAufAusgewaehlte(self):
        if self.__ausgewaehlt==None: return
        for obj in self.__klassenkarten+self.__assoziationen+self.__vererbungen:
            obj.Verberge()
        zf=Zeichenflaeche.GibZeichenflaeche()
        zf.SetzeSkalierung(2) # Klassenkarte vergroessern
        a=self.__ausgewaehlt
        zf.SetzeVerschiebung(a.GibX()-20,a.GibY()-20) # Klassenkarte verschieben
        self.__ausgewaehlt.Zeige()

    def ZeigeAlle(self):
        Zeichenflaeche.GibZeichenflaeche().SetzeSkalierung(1)
        for obj in self.__klassenkarten+self.__assoziationen+self.__vererbungen:
            obj.Zeige()

    def LoescheAusgewaehlte(self):
        '''es werden ausser der Klasse auch die
        zugehoerigen Verbinder geloescht'''
        karte = self.__ausgewaehlt
        zuLoeschen=[]
        for asso in self.__assoziationen:
            if karte in (asso.GibStartKarte(),
                         asso.GibZielKarte()):
                zuLoeschen.append(asso)
        for asso in zuLoeschen:
            asso.Verberge()
            self.__assoziationen.remove(asso)
        zuLoeschen=[]
        for vererbung in self.__vererbungen:
            if karte in [vererbung.GibStartKarte(),
                         vererbung.GibZielKarte()]:
                zuLoeschen.append(vererbung)
        for vererbung in zuLoeschen:
            vererbung.Verberge()
            self.__vererbungen.remove(vererbung)
        karte.Verberge()
        self.__klassenkarten.remove(karte)
        self.Waehle(None)
        self.__geaendert=True

    def UpdateGrafikfenster(self):
        karte = self.__ausgewaehlt
        self.__geaendert=True
        for kd_verb in self.__assoziationen + self.__vererbungen:
            if kd_verb.GibStartKarte()==karte:
                kd_verb.UpdateVerbinder()
            if kd_verb.GibZielKarte()==karte:
                kd_verb.UpdateVerbinder()

    ### ----- Export py-code -----------------------------
    def SpeichereCode(self, dateiname):
        if self.NichtsAusgewaehlt:
            karten=self.__klassenkarten
        else:
            karten=[self.GibAusgewaehlte()]
        gesamtcode=""
        for karte in karten:
            gesamtcode+=self.ErzeugeCode(karte)
            gesamtcode+="\n\n"
        try:
            f=open(dateiname,'w')
            f.write(gesamtcode)
        except IOError as e:
            self.__gui.Ausgabe('Ausgabefehler')
            return None
        f.close()
        return 'OK'
    
    def ErzeugeCode(self, karte):
        codeListe=[]
        if karte.IstAbstrakt():
            codeListe.append(
                "from abc import ABC,abstractmethod # abstrakte Klasse\n\n")
        codeListe+=self.KlassenKopf(karte, self.KlasseErbtVon(karte))
        codeListe.append("\n")
        cv=self.ClassvarCode(karte.GibAttribute())
        if cv!=[]:
            codeListe+=cv
            codeListe.append("\n")
        cm=self.ClassmethCode(karte.GibMethoden())
        if cm!=[]:
            codeListe+=cm
            codeListe.append("\n")
        codeListe.append(self.KonstruktorKopf(karte.GibMethoden()))
        codeListe.append(self.KonstruktorenDerOberklassen(karte))
        codeListe+=self.AttributeCode(karte.GibAttribute())
        codeListe+=self.MethodenCode(karte.GibMethoden())
        ##print(codeListe)
        code=""
        for teil in codeListe:
            code+=teil
        return code
    
    def KlassenKopf(self, karte, erbtVon=[]):
        codeListe=[]
        if karte.IstAbstrakt():
            codeListe.append("class "+
                             karte.GibNamen()+"("+
                             "ABC")
        else:
            codeListe.append("class "+karte.GibNamen()+"(")
        for oberklasse in erbtVon:
            if (len(codeListe)>1) or karte.IstAbstrakt():
                codeListe.append(", ")
            codeListe.append(oberklasse.GibNamen())
        codeListe.append("):\n")
        return codeListe

    def ClassvarCode(self, attribute):
        codeListe=[]
        for attr in attribute:
            if len(attr)>2:
                if attr[2]:
                    codeListe.append(self.EinR(1)+
                                     self.AttrKonstr(attr)[5:] # self. raus
                                     +"\n")
        return codeListe

    def ClassmethCode(self, methoden):
        codeListe=[]
        for meth in methoden:
            if len(meth)>4:
                if meth[4]:
                    code,doc=self.ErzeugeMethode(meth)
                    code=self.EntferneSelf(code)
                    code=self.EinR(1)+"@classmethod\n"+code
                    codeListe.append(code)
                    codeListe.append(self.EinR(2)+"'''"+doc+"'''"+"\n")
                    codeListe.append(self.EinR(2)+"pass"+"\n\n")
        return codeListe

    def KonstruktorKopf(self, methoden):
        '''Wenn der Kontruktor __init__ unter den Methoden nicht
        angegeben ist, wird ein Standardkonstruktor generiert'''
        konstruktor="def __init__(self"
        for methode in methoden:
            if methode[0]=="__init__":
                for par in methode[2]:
                    konstruktor+=", "
                    konstruktor+=par[0]+':'+par[1]
                break
        return self.EinR(1)+konstruktor+"):\n"
    
    def KonstruktorenDerOberklassen(self, karte):
        karten=self.KlasseErbtVon(karte)
        aufrufe=""
        for oberklasse in karten:
            aufrufe+=(self.EinR(2)+oberklasse.GibNamen()+
                      ".__init__(self, <parameter ergaenzen>)\n")
        return aufrufe

    def AttributeCode(self, attribute):
        codeListe=[]
        for attr in attribute:
            if (len(attr)<=2) or (not(attr[2])):
                #print(self.AttrKonstr(attr))
                codeListe.append(self.EinR(2)+
                                 self.AttrKonstr(attr)+"\n")
        if codeListe==[]: codeListe=[self.EinR(2)+"pass"]
        return codeListe

    def MethodenCode(self, methoden):
        ##print('In MethodenCode',methoden)
        codeListe=["\n"]
        for meth in methoden:
            if meth[0]=="__init__": continue
            if (len(meth)<=4) or (not(meth[4])):
                code,doc=self.ErzeugeMethode(meth)
                codeListe.append(code)
                codeListe.append(self.EinR(2)+"'''"+doc+"'''"+"\n")
                codeListe.append(self.EinR(2)+"pass"+"\n\n")
        return codeListe

    ### Hilfsmethoden dazu
    def KlasseErbtVon(self, karte):
        oberklassen=[]
        for istEin in self.__vererbungen:
            if istEin.GibStartKarte()==karte:
                oberklassen.append(istEin.GibZielKarte())
        return oberklassen
    
    def EinR(self, n):
        '''gibt n Einrueckungen zurueck'''
        einrueckung=""
        for i in range(n):
            einrueckung+="    "
        return einrueckung

    def AttrKonstr(self, attribut):
        d={'int':'0', 'float':'0.0', 'bool':'False', 'String':''}
        initwert='None'
        if attribut[1] in d:
            initwert=d[str(attribut[1])]
        return "self."+attribut[0]+"="+initwert+"  # "+attribut[1]

    def ErzeugeMethode(self, methode):
        code=self.EinR(1)+"def "+methode[0]+"(self"
        doc=""
        if methode[2]!=[]:
            undCode,doc=self.ErzeugeParameter(methode[2])
            code+=undCode
        code+="):\n"
        return code, doc

    def ErzeugeParameter(self, parameterListe):
        # vordefinierte Parameter-Werte fehlen
        code=""
        doc=""
        for par in parameterListe:
            code+=", "
            code+=par[0]
            if par!=parameterListe[0]:doc+=", "
            doc+=par[0]+':'+par[1]
        return code, doc

    def EntferneSelf(self, code):
        if code.find('self, ')!=-1:
            teile=code.partition('self, ')
            return teile[0]+teile[2]
        else:
            teile=code.partition('self')
            return teile[0]+teile[2]

    def SetzeCodePage(self):
        '''Bei Py3 sinnlos'''
        codeListe=["-*- coding: utf-8 -*-\n\n"]
        return codeListe
